%==========================================================================
% Main script
%==========================================================================

addpath('./aux_tools')
addpath('./mtimesx')

%% Global parameters

dataDir = './sounds'; % directory of true source images and mix 
resultsDir = dataDir; % directory to write estimated source images
resultsPrefix = 'sim_VEM_MDCT_';

J = 5; % number of sources
I = 2; % number of microphones
wlen = 2048;
hop = wlen/2;
niter = 200; % number of iterations for the GEM algorithm

wavFile = fullfile(dataDir,'mix.wav'); 
[x, fs] = audioread(wavFile);
X = mdct_multi(x, wlen);

T_orig = size(x,1);
T60 = 256; % reverberation time in ms
La = T60*1e-3*fs;
Ls = T_orig-La+1;

%% Zero padding for handling edges

Nzp_end = ceil(Ls/hop)*hop-Ls;
Nzp_end = Nzp_end + wlen/2;
Nzp_beg = wlen/2;
Ls = Ls + Nzp_beg + Nzp_end;

x = [zeros(Nzp_beg, I) ; x ; zeros(Nzp_end, I)];
T = size(x,1);

F = wlen/2;
N = round(Ls/hop) - 1; % number of frames
win = sin((.5:wlen-.5)/wlen*pi).';

%% Initialization

st = load(fullfile(dataDir, 'mix_filters.mat'));
a_init = st.A;

K = 10; % NMF order
mix_psd = 0.5 * (mean(abs(X(:,:,1)).^2 + abs(X(:,:,2)).^2, 2));
W_init = cell(J,1);
H_init = cell(J,1);
K = 10; % NMF order
for j=1:J
    rng(1);
    W_init{j} = 0.5 * (abs(randn(F,K)) + ones(F,K)) .* (mix_psd * ones(1,K));
    rng(1);
    H_init{j} = 0.5 * (abs(randn(K,N)) + ones(K,N));
end

sigmaNoise_init = sum(abs(x).^2,1)/T;

%% Run VEM algorithm

fprintf('Instantiation of VEMalgo\n');
ss_algoVEM = VEMalgo('x', x, 'a', a_init, 'sigmaNoise', sigmaNoise_init, 'W', {W_init}, 'H', {H_init});

varFreeEnergy = zeros(niter,1);

for iter = 1:niter
    fprintf('\nVEM iteration %d of %d\n\n', iter, niter);
    
    ss_algoVEM.EStepPCG;
    
    fprintf('Update NMF\n');
    ss_algoVEM.UpdateNMF;
    
    fprintf('Compute expected error\n');
    ss_algoVEM.ComputeExpectError;
    
    fprintf('Update Noise\n');
    ss_algoVEM.UpdateNoiseVar;
    
    ss_algoVEM.ComputeVarFreeEnergy(0);
    varFreeEnergy(iter) = ss_algoVEM.varFreeEnergy;

    fprintf('variational free energy: %.2f\n', varFreeEnergy(iter)/(I*T));
    
    
end
ss_algoVEM.EStepPCG;


%% Compute estimated source images

Ls = T_orig-La+1;
se = imdct_multi(ss_algoVEM.m_post, Ls);
sime = zeros(J, T_orig, I);
for j=1:J
    for i=1:I
    sime(j,:,i) = conv(se(:,j), squeeze(ss_algoVEM.a(i,j,:)));
    end
end

x_orig = x(Nzp_beg+1:Nzp_beg+T_orig,:);

for k=1:J,
    audiowrite(fullfile(resultsDir, [resultsPrefix num2str(k) '.wav']), reshape(sime(k,:,:),T_orig,2),fs);
end